#import "CircleTextView.h"

#import "CTSentenceFormatter.h"

NSString * const CT_CIRCLES_MAINSCREEN_ONLY=@"MainScreen Only";

NSString * const CT_CIRCLES_DEFINITIONS_KEY=@"circles.definitions";

NSString * const CT_CIRCLES_DEFINITIONS_TEXT=@"text";

NSString * const CT_CIRCLES_DEFINITIONS_SPEED=@"speed";

NSString * const CT_CIRCLES_DEFINITIONS_SCALE=@"scale";

NSString * const CT_CIRCLES_DEFINITIONS_RADIUS=@"radius";

NSString * const CT_CIRCLES_DEFINITIONS_X_OFFSET=@"x";

NSString * const CT_CIRCLES_DEFINITIONS_Y_OFFSET=@"y";

@interface NSArray (CTMutableDeepCopy)

- (id) mutableDeepCopy;

@end

@implementation NSArray (CTMutableDeepCopy)

- (id) mutableDeepCopy
{
	NSData * tData;
	NSString * tErrorString;
	
	tData=[NSPropertyListSerialization dataFromPropertyList:self
													 format:NSPropertyListXMLFormat_v1_0
										   errorDescription:&tErrorString];
	
	if (tData!=nil)
	{
		NSMutableArray * tMutableArray;
		NSPropertyListFormat tFormat;
		
		tMutableArray=(NSMutableArray *) [NSPropertyListSerialization propertyListFromData:tData
																					mutabilityOption:NSPropertyListMutableContainersAndLeaves
																							  format:&tFormat
																					errorDescription:&tErrorString];
		
		if (tMutableArray==nil)
		{
			NSLog(@"[NSArray mutableDeepCopy] : %@",tErrorString);
			
			[tErrorString release];
		}
		
		return [tMutableArray retain];
	}
	else
	{
		NSLog(@"[NSArray mutableDeepCopy] : %@",tErrorString);
		
		[tErrorString release];
	}
	
	return nil;
}

@end

@implementation CircleTextView

+ (circ_def *) circleDefinitionsFromArray:(NSArray *) inArray count:(int *) outCount
{
	circ_def * nCircleDefintions=NULL;
	int tCount;
	
	tCount=(int) [inArray count];
	
	if (tCount>0)
	{
		nCircleDefintions=(circ_def *) malloc(tCount*sizeof(circ_def));
		
		if (nCircleDefintions!=NULL)
		{
			int i;
			
			*outCount=tCount;
			
			for(i=0;i<tCount;i++)
			{
				NSDictionary * tCircleDictionary;
				NSString * tString;
				
				tCircleDictionary=[inArray objectAtIndex:i];
				
				tString=[tCircleDictionary objectForKey:CT_CIRCLES_DEFINITIONS_TEXT];
				
				if ([tString length]>0)
				{
					if ([tString getCString:nCircleDefintions[i].text
								  maxLength:(MAX_TEXT_LEN-1) 
								   encoding:NSASCIIStringEncoding]==NO)
					{
						nCircleDefintions[i].text[0]='\0';
					}
				}
				else
				{
					nCircleDefintions[i].text[0]='\0';
				}

				nCircleDefintions[i].x=[[tCircleDictionary objectForKey:CT_CIRCLES_DEFINITIONS_X_OFFSET] intValue];
				nCircleDefintions[i].y=[[tCircleDictionary objectForKey:CT_CIRCLES_DEFINITIONS_Y_OFFSET] intValue];
				
				nCircleDefintions[i].speed=[[tCircleDictionary objectForKey:CT_CIRCLES_DEFINITIONS_SPEED] intValue];
				
				nCircleDefintions[i].scale=[[tCircleDictionary objectForKey:CT_CIRCLES_DEFINITIONS_SCALE] intValue];
				
				nCircleDefintions[i].radius=[[tCircleDictionary objectForKey:CT_CIRCLES_DEFINITIONS_RADIUS] intValue];
			}
		}
	}
	else
	{
		*outCount=0;
	}
	
	return nCircleDefintions;
}

+ (NSArray *) arrayWithCircleDef:(circ_def *) inCircleDefs count:(int) inCount
{
	NSMutableArray * tMutableArray=nil;
	
	if (inCircleDefs!=NULL && inCount>0)
	{
		tMutableArray=[NSMutableArray arrayWithCapacity:inCount];
		
		if (tMutableArray!=nil)
		{
			int i;
			
			for(i=0;i<inCount;i++)
			{
				NSDictionary * tDictionary;
				NSString * tString;
				
				tString=[NSString stringWithCString:inCircleDefs[i].text encoding:NSASCIIStringEncoding];
				
#if __LITTLE_ENDIAN__
				
				inCircleDefs[i].x=CFSwapInt32(inCircleDefs[i].x);
				inCircleDefs[i].y=CFSwapInt32(inCircleDefs[i].y);
				inCircleDefs[i].speed=CFSwapInt32(inCircleDefs[i].speed);
				inCircleDefs[i].scale=CFSwapInt32(inCircleDefs[i].scale);
				inCircleDefs[i].radius=CFSwapInt32(inCircleDefs[i].radius);

#endif
				
				tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tString,CT_CIRCLES_DEFINITIONS_TEXT,
																	   [NSNumber numberWithInt:inCircleDefs[i].x],CT_CIRCLES_DEFINITIONS_X_OFFSET,
																	   [NSNumber numberWithInt:inCircleDefs[i].y],CT_CIRCLES_DEFINITIONS_Y_OFFSET,
																	   [NSNumber numberWithInt:inCircleDefs[i].speed],CT_CIRCLES_DEFINITIONS_SPEED,
																	   [NSNumber numberWithInt:inCircleDefs[i].scale],CT_CIRCLES_DEFINITIONS_SCALE,
																	   [NSNumber numberWithInt:inCircleDefs[i].radius],CT_CIRCLES_DEFINITIONS_RADIUS,
																	   nil];
				
				if (tDictionary!=nil)
				{
					[tMutableArray addObject:tDictionary];
				}
			}
		}
	}
	
	return tMutableArray;
}

#pragma mark -

- (id) initWithFrame:(NSRect) inFrame isPreview:(BOOL) inPreview
{
    NSString * tdentifier;
    ScreenSaverDefaults * tDefaults;
    
	bundle_=[NSBundle bundleForClass:[self class]];
	
	tdentifier = [bundle_ bundleIdentifier];
	
	tDefaults = [ScreenSaverDefaults defaultsForModuleWithName:tdentifier];
	
    if (![super initWithFrame:inFrame isPreview:inPreview]) return nil;
	
    preview_=inPreview;
    
    isConfiguring_=NO;
    
    if (preview_==YES)
    {
        mainScreen_=YES;
    }
    else
    {
        mainScreen_= (inFrame.origin.x==0 && inFrame.origin.y==0) ? YES : NO;
    }
    
    mainScreenOnly_=[tDefaults integerForKey:CT_CIRCLES_MAINSCREEN_ONLY];
    
    if (self)
    {
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            NSOpenGLPixelFormatAttribute attribs[] =
            {
                    NSOpenGLPFADepthSize, 16,
                    NSOpenGLPFAMinimumPolicy,
                    NSOpenGLPFAClosestPolicy,
                    0
            };
            
            NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
            
            if (format!=nil)
            {
                _view = [[NSOpenGLView alloc] initWithFrame:NSZeroRect pixelFormat:format];
				
				if (_view!=nil)
				{
					id tObject;
					NSData * tOldData;
					
					[self addSubview:_view];
					
					[_view release];
					
					if (fontImageData==NULL)
					{
						NSData * tData;
						
						tData=[NSData dataWithContentsOfFile:[bundle_ pathForResource:@"FONT" ofType:@"RAW"]];
					
						if (tData!=nil)
						{
							// IMPORTANT: CFRetain is used to avoid the GC to release the bytes buffer.
							
							CFRetain(tData);
							
							fontImageData=(unsigned char *) [tData bytes];
						}
					}
					
					// Check if we have new data
					
					tObject=[tDefaults objectForKey:CT_CIRCLES_DEFINITIONS_KEY];
					
					if (tObject==nil)
					{
						// Check if we have some old data
						
						tOldData=[tDefaults objectForKey:@"data"];
					
						if (tOldData!=nil)
						{
							// Convert the old data
							
							circ_def * tCircleDefArray;
							int tCircleCount;
							
							tCircleCount=[tOldData length]/sizeof(circ_def);
							
							tCircleDefArray=(circ_def *) malloc(tCircleCount*sizeof(circ_def));
							
							if (tCircleDefArray!=NULL)
							{
								NSArray * tArray;
								
								memcpy(tCircleDefArray,[tOldData bytes],tCircleCount*sizeof(circ_def));
							
								tArray=[CircleTextView arrayWithCircleDef:tCircleDefArray count:tCircleCount];
								
								[tDefaults setObject:tArray forKey:CT_CIRCLES_DEFINITIONS_KEY];
							}
							
							[tDefaults removeObjectForKey:@"data"];
						}
					}
				}
			}
		}
	}
	
    return self;
}

- (void) setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    
    if (_view!=nil)
    {
        [_view setFrameSize:newSize];
    }
}

- (void) drawRect:(NSRect) inFrame
{
	[[NSColor blackColor] set];
            
    NSRectFill(inFrame);
    
    if (_view==nil)
    {    
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            NSRect tFrame=[self frame];
            NSRect tStringFrame;
            NSDictionary * tAttributes;
            NSString * tString;
            NSMutableParagraphStyle * tParagraphStyle;
            
            tParagraphStyle=[[NSParagraphStyle defaultParagraphStyle] mutableCopy];
            [tParagraphStyle setAlignment:NSCenterTextAlignment];
            
            tAttributes = [NSDictionary dictionaryWithObjectsAndKeys:[NSFont systemFontOfSize:[NSFont systemFontSize]],NSFontAttributeName,[NSColor whiteColor],NSForegroundColorAttributeName,tParagraphStyle,NSParagraphStyleAttributeName,nil];
            
            [tParagraphStyle release];
            
            tString=NSLocalizedStringFromTableInBundle(@"Minimum OpenGL requirements\rfor this Screen Effect\rnot available\ron your graphic card.",@"Localizable",bundle_,@"No comment");
            
            tStringFrame.origin=NSZeroPoint;
            tStringFrame.size=[tString sizeWithAttributes:tAttributes];
            
            tStringFrame=SSCenteredRectInRect(tStringFrame,tFrame);
            
            [tString drawInRect:tStringFrame withAttributes:tAttributes];
            
            return;
        }
    }
}

#pragma mark -

- (void) startAnimation
{
    [super startAnimation];
    
    if (_view!=nil)
    {
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            NSSize tSize;
			NSString * tdentifier;
			ScreenSaverDefaults * tDefaults;
			NSArray * tArray;
			
			bundle_=[NSBundle bundleForClass:[self class]];
			
			tdentifier = [bundle_ bundleIdentifier];
			
			tDefaults = [ScreenSaverDefaults defaultsForModuleWithName:tdentifier];
			
			// Do your animation initialization here
            
            [self lockFocus];
            [[_view openGLContext] makeCurrentContext];
                
            tSize=[self frame].size;
			
			resize(tSize.width, tSize.height);
			
			// Read Settings
			
			tArray=[tDefaults objectForKey:CT_CIRCLES_DEFINITIONS_KEY];
			
			if (tArray==nil)
			{
				// Load the array from the default.plist
				
				tArray=[NSArray arrayWithContentsOfURL:[NSURL fileURLWithPath:[bundle_ pathForResource:@"DefaultSettings" ofType:@"plist"]]];
			}
			
			circleDefinitionsArray_=[CircleTextView circleDefinitionsFromArray:tArray count:&circleDefinitionsCount_];
			
			ss_initialize();
                
			ss_draw(circleDefinitionsArray_,circleDefinitionsCount_,[[NSDate date] timeIntervalSince1970]);
            
            [self unlockFocus];
        }
    }
}

- (void) animateOneFrame
{
    if (_view!=nil)
    {
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            [[_view openGLContext] makeCurrentContext];
			
           if (circleDefinitionsArray_!=NULL)
		   {
			   ss_draw(circleDefinitionsArray_,circleDefinitionsCount_,[[NSDate date] timeIntervalSince1970]);
		   }
		}
    }
}

- (void) stopAnimation
{
    [super stopAnimation];
    
	if (circleDefinitionsArray_!=NULL)
	{
		free(circleDefinitionsArray_);
	
		circleDefinitionsArray_=NULL;
	}
	
	circleDefinitionsCount_=0;
	
    if (_view!=nil)
    {
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            [[_view openGLContext] makeCurrentContext];
        }
    }
}

#pragma mark -

- (BOOL) hasConfigureSheet
{
    return (_view!=nil);
}

- (NSWindow *) configureSheet
{
    NSString * tIdentifier = [bundle_ bundleIdentifier];
	ScreenSaverDefaults * tDefaults = [ScreenSaverDefaults defaultsForModuleWithName:tIdentifier];
	
	cachedDictionary_=nil;
	
	if (configureSheet_ == nil)
    {
        CTSentenceFormatter * tFormatter;
		
		[NSBundle loadNibNamed:@"CircleText" owner:self];
		
		tFormatter=[CTSentenceFormatter new];
		
		if (tFormatter!=nil)
		{
			NSTableColumn * tTableColumn;
			NSTextFieldCell * tTextFieldCell;
			
			tTableColumn=[IBarray_ tableColumnWithIdentifier: @"sentence"];
			
			tTextFieldCell=[tTableColumn dataCell];
			
			[tTextFieldCell setFormatter:tFormatter];
			
			[tFormatter release];
		}
		
		// Set the Plus and Minus icons

		if (NSAppKitVersionNumber>=NSAppKitVersionNumber10_5)
		{
			[IBadd_ setImage:[NSImage imageNamed:@"NSAddTemplate"]];
		
			[IBremove_ setImage:[NSImage imageNamed:@"NSRemoveTemplate"]];
		}
		else
		{
			NSImage * tImage;
			
			tImage=[[NSImage alloc] initWithContentsOfFile:[bundle_ pathForResource:@"NSAddTemplateForTiger" ofType:@"tiff"]];
			
			if (tImage!=nil)
			{
				[IBadd_ setImage:tImage];
				
				[tImage release];
			}
			
			tImage=[[NSImage alloc] initWithContentsOfFile:[bundle_ pathForResource:@"NSRemoveTemplateForTiger" ofType:@"tiff"]];
			
			if (tImage!=nil)
			{
				[IBremove_ setImage:tImage];
				
				[tImage release];
			}
		}

    }
	currentView_=nil;
	
    // Init values
    
	// Read the settings
	
	circlesArray_=[[tDefaults objectForKey:CT_CIRCLES_DEFINITIONS_KEY] mutableDeepCopy];
	
	if (circlesArray_==nil)
	{
		// Load the array from the default.plist
			
		circlesArray_=[[NSArray arrayWithContentsOfURL:[NSURL fileURLWithPath:[bundle_ pathForResource:@"DefaultSettings" ofType:@"plist"]]] mutableDeepCopy];
	}
	
    [IBmainScreen_ setState:mainScreenOnly_];
    
    [IBremove_ setEnabled:NO];
    
	[IBarray_ deselectAll:self];
	
	[IBarray_ reloadData];
	
	[self tableViewSelectionDidChange:nil];
    
    return configureSheet_;
}

- (IBAction) endDialog:(id) sender
{
    if ([sender tag]==NSOKButton)
    {
		NSString * tIdentifier = [bundle_ bundleIdentifier];
		ScreenSaverDefaults * tDefaults = [ScreenSaverDefaults defaultsForModuleWithName:tIdentifier];
		
		// Save Settings
		
		[tDefaults setObject:circlesArray_ forKey:CT_CIRCLES_DEFINITIONS_KEY];
		
		mainScreenOnly_=[IBmainScreen_ state];
		
		[tDefaults setInteger:mainScreenOnly_ forKey:CT_CIRCLES_MAINSCREEN_ONLY];
    }
	
	[circlesArray_ autorelease];
	
	circlesArray_=nil;
	
	cachedDictionary_=nil;
	
	[currentView_ removeFromSuperview];
	
	currentView_=nil;
	
	// For SaverLab mainly
	
	if ([self isAnimating]==YES)
    {
        [self stopAnimation];
        [self startAnimation];
    }
	
    [NSApp endSheet:configureSheet_];
}

#pragma mark -

- (IBAction) addSentence:(id) sender
{
	NSMutableDictionary * tMutableDictionary;
	
	tMutableDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedStringFromTableInBundle(@"default sentence", @"CircleText", bundle_, @"No comment"),CT_CIRCLES_DEFINITIONS_TEXT,
																		 [NSNumber numberWithInt:50],CT_CIRCLES_DEFINITIONS_SPEED,
																		 [NSNumber numberWithInt:4000],CT_CIRCLES_DEFINITIONS_SCALE,
																		 [NSNumber numberWithInt:200],CT_CIRCLES_DEFINITIONS_RADIUS,
																		 [NSNumber numberWithInt:50],CT_CIRCLES_DEFINITIONS_X_OFFSET,
																		 [NSNumber numberWithInt:50],CT_CIRCLES_DEFINITIONS_Y_OFFSET,
																		 nil];
	
	if (tMutableDictionary!=nil)
	{
		NSInteger tCount;
		
		tCount=[circlesArray_ count];
		
		[circlesArray_ addObject:tMutableDictionary];
		
		[IBarray_ reloadData];
		
		[IBarray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tCount] byExtendingSelection:NO];
		
		[IBarray_ editColumn:[IBarray_ columnWithIdentifier:@"sentence"] row:tCount withEvent:nil select:YES];
		
		cachedDictionary_=tMutableDictionary;
		
		[self showValuesForRow:tCount];
	}
	else
	{
		NSBeep();
	}
}

- (IBAction) removeSentence:(id) sender
{
    NSString * tString;
    
    if ([IBarray_ numberOfSelectedRows]==1)
    {
        tString=NSLocalizedStringFromTableInBundle(@"Do you really want to remove this sentence?", @"CircleText", bundle_, @"No comment");
    }
    else
    {
        tString=NSLocalizedStringFromTableInBundle(@"Do you really want to remove these sentences?", @"CircleText", bundle_, @"No comment");
    }
    
    NSBeginAlertSheet(tString,
                      NSLocalizedStringFromTableInBundle(@"Remove", @"CircleText", bundle_, @"No comment"),
                      NSLocalizedStringFromTableInBundle(@"Cancel", @"CircleText", bundle_, @"No comment"),
                      nil,
                      [IBarray_ window],
                      self,
                      @selector(removeSheetDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedStringFromTableInBundle(@"This can not be undone", @"CircleText", bundle_, @"No comment"));
}

- (void) removeSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger) inReturnCode contextInfo:(void *) contextInfo
{
    if (inReturnCode==NSOKButton)
    {
        NSIndexSet * tIndexSet;
		
		cachedDictionary_=nil;
		
		tIndexSet=[IBarray_ selectedRowIndexes];
		
		[circlesArray_ removeObjectsAtIndexes:tIndexSet];
		
		[IBarray_ deselectAll:nil];
		
		[IBarray_ reloadData];
		
		[self tableViewSelectionDidChange:nil];
    }
}

- (void) showValuesForRow:(NSInteger) inRowIndex
{
	if (cachedDictionary_!=nil)
	{
		[IBradius_ setFloatValue:[[cachedDictionary_ objectForKey:CT_CIRCLES_DEFINITIONS_RADIUS] intValue]];
    
		[IBscale_ setFloatValue:[[cachedDictionary_ objectForKey:CT_CIRCLES_DEFINITIONS_SCALE] intValue]];
    
		[IBrotationSpeed_ setFloatValue:(-0.2*[[cachedDictionary_ objectForKey:CT_CIRCLES_DEFINITIONS_SPEED] intValue]+11)];
    
		[IBx_ setFloatValue:[[cachedDictionary_ objectForKey:CT_CIRCLES_DEFINITIONS_X_OFFSET] intValue]];
    
		[IBy_ setFloatValue:[[cachedDictionary_ objectForKey:CT_CIRCLES_DEFINITIONS_Y_OFFSET] intValue]];
	}
}

#pragma mark -

- (IBAction) setRadius:(id) sender
{
    [cachedDictionary_ setObject:[NSNumber numberWithFloat:[IBradius_ floatValue]] forKey:CT_CIRCLES_DEFINITIONS_RADIUS];
}

- (IBAction) setScale:(id) sender
{
    [cachedDictionary_ setObject:[NSNumber numberWithFloat:[IBscale_ floatValue]] forKey:CT_CIRCLES_DEFINITIONS_SCALE];
}

- (IBAction) setRotationSpeed:(id) sender
{
    [cachedDictionary_ setObject:[NSNumber numberWithFloat:(-5*[IBrotationSpeed_ floatValue]+55)] forKey:CT_CIRCLES_DEFINITIONS_SPEED];
}

- (IBAction) setX:(id) sender
{
    [cachedDictionary_ setObject:[NSNumber numberWithFloat:[IBx_ floatValue]] forKey:CT_CIRCLES_DEFINITIONS_X_OFFSET];
}

- (IBAction) setY:(id) sender
{
    [cachedDictionary_ setObject:[NSNumber numberWithFloat:[IBy_ floatValue]] forKey:CT_CIRCLES_DEFINITIONS_Y_OFFSET];
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (inTableView==IBarray_)
	{
		return [circlesArray_ count];
	}

	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
    if (inTableView==IBarray_)
	{
		NSDictionary * tDictionary;
		
		tDictionary=[circlesArray_ objectAtIndex:inRowIndex];
		
		if (tDictionary!=nil)
		{
			NSString * tIdentifier;
			
			tIdentifier=[inTableColumn identifier];
			
			if ([tIdentifier isEqualToString:@"sentence"]==YES)
			{
				return [tDictionary objectForKey:CT_CIRCLES_DEFINITIONS_TEXT];
			}
		}
	}
		
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) inObject forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (inTableView==IBarray_)
	{
		NSString * tIdentifier;
		
		tIdentifier=[inTableColumn identifier];
		
		if ([tIdentifier isEqualToString:@"sentence"]==YES)
		{
			NSString * tNewString;
			
			tNewString=(NSString *) inObject;
			
			if (strlen([tNewString UTF8String])>=MAX_TEXT_LEN)
			{
				NSBeep();
			}
			else
			{
				NSMutableDictionary * tMutableDictionary;
				
				tMutableDictionary=[circlesArray_ objectAtIndex:inRowIndex];
				
				if (tMutableDictionary!=nil)
				{
					[tMutableDictionary setObject:tNewString forKey:CT_CIRCLES_DEFINITIONS_TEXT];
				}
			}
		}
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([IBarray_ numberOfSelectedRows]==0)
    {
        [IBremove_ setEnabled:NO];
        
        // Show Empty selection pane
		
		if (currentView_!=IBemptySelectionView_)
		{
			[currentView_ removeFromSuperview];
			
			[IBbox_ addSubview:IBemptySelectionView_];
			
			currentView_=IBemptySelectionView_;
		}
		
		cachedDictionary_=nil;
    }
    else
    {
        [IBremove_ setEnabled:YES];
        
        if ([IBarray_ numberOfSelectedRows]==1)
        {
            NSInteger tSelectedRow;
			
			// Show Single Selection Pane (if needed)
			
			if (currentView_!=IBsettingsView_)
			{
				[currentView_ removeFromSuperview];
				
				[IBbox_ addSubview:IBsettingsView_];
				
				currentView_=IBsettingsView_;
			}
			
			// Show values for selected sentence
			
			tSelectedRow=[IBarray_ selectedRow];
			
			cachedDictionary_=[circlesArray_ objectAtIndex:tSelectedRow];
			
			[self showValuesForRow:tSelectedRow];
        }
        else
		{
			cachedDictionary_=nil;
			
			// Show Multiple selection pane
			
			if (currentView_!=IBmultipleSelectionView_)
			{
				[currentView_ removeFromSuperview];
				
				[IBbox_ addSubview:IBmultipleSelectionView_];
				
				currentView_=IBmultipleSelectionView_;
			}
			
        }
    }
}

@end
