#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include "pawnres.h"
#include "font.h"
#include <time.h>
#include "Pawn.h"
#include <strings.h>

int options=HIGH_QUALITY | SHOW_LOGO;

unsigned char bgcolor[3]={0,0,0};

unsigned char * fontImageData=NULL;

/* new window size or exposure */

#pragma pack(4)

typedef struct
{
  float tx1, ty1, tx2, ty2;
  float width;
} char_def;

#pragma pack()

char_def font_table[FONT_LETTERS];

GLuint texture_names[TEXTURE_COUNT];

void init_font(void)
{
  int i, length=0;
  float avg_width;

  for(i=0;i<26;i++)
    length+=font_pos_table[i].right-font_pos_table[i].left;

  avg_width=(float)length/(float)26;

  for(i=0;i<FONT_LETTERS;i++)
  {
    font_table[i].tx1=(float)font_pos_table[i].left/(float)FONT_IMG_WIDTH;
    font_table[i].tx2=(float)font_pos_table[i].right/(float)FONT_IMG_WIDTH;
    font_table[i].ty1=(float)font_pos_table[i].top/(float)FONT_IMG_HEIGHT;
    font_table[i].ty2=(float)font_pos_table[i].bottom/(float)FONT_IMG_HEIGHT;

    font_table[i].width=(float)(font_pos_table[i].right-font_pos_table[i].left)/avg_width;
  }
}

char_def *get_char_def(char a)
{
  if(isalpha(a))
  {
    if(isupper(a)) return font_table+(a-'A');
    else return font_table+26+(a-'a');
  }
  else if(a=='.') return font_table+26*2;
  else if(a==',') return font_table+26*2+1;
  else return NULL;
}

void draw_letter(char letter)
{
  float width, aspect=.7;
  char_def *cd;

  cd=get_char_def(letter);
  if(!cd) return;

  width=cd->width*aspect;
  glBegin(GL_POLYGON);
  glTexCoord2f(cd->tx1, cd->ty2);
  glVertex3f(-width/2.0,-.5,0);
  glTexCoord2f(cd->tx2, cd->ty2);
  glVertex3f(width/2.0,-.5,0);
  glTexCoord2f(cd->tx2, cd->ty1);
  glVertex3f(width/2.0,.5,0);
  glTexCoord2f(cd->tx1, cd->ty1);
  glVertex3f(-width/2.0,.5,0);
  glEnd();
}

float draw_string_len(char *s)
{
  float len=0;
  char_def *cd;

  while(*s)
  {
    cd=get_char_def(*s);
    if(cd)
    {
      len+=cd->width;
    }
  }
  return len;
}

void predraw_string(void)
{
  glPushAttrib(GL_COLOR_BUFFER_BIT);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
  glBindTexture(GL_TEXTURE_2D, FONT_TEXTURE);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

void postdraw_string(void)
{
  glPopAttrib();
}

void draw_string(char *s, float x, float y)
{
	float scale=30;
	float aspect=.7;
	float width;
	char_def *cd;

	while(*s)
	{
		cd=get_char_def(*s);
		if(cd)
		{
			width=scale*cd->width*aspect;
			glBegin(GL_POLYGON);
			glTexCoord2f(cd->tx1, cd->ty2);
			glVertex3f(x,y,0);
			glTexCoord2f(cd->tx2, cd->ty2);
			glVertex3f(x+width,y,0);
			glTexCoord2f(cd->tx2, cd->ty1);
			glVertex3f(x+width,y+scale,0);
			glTexCoord2f(cd->tx1, cd->ty1);
			glVertex3f(x,y+scale,0);
			glEnd();
			x+=width;
		}
		s++;
	}
}


void load_texture(int name, unsigned char *texture, int tex_width, int tex_height, int type);

void ss_initialize()
{
    //LoadConfig();

    glGenTextures(TEXTURE_COUNT, texture_names);

#define GET_RESDATA(x) LockResource(LoadResource(NULL, FindResource(NULL, x, RT_RCDATA)))

	//  load_texture(MOON_TEXTURE, GET_RESDATA("moonimg"), 1024, 512, GL_LUMINANCE);
	//load_texture(LOGO_TEXTURE, GET_RESDATA("logoimg"), 256, 128, GL_RGB);

	load_texture(FONT_TEXTURE, fontImageData, FONT_IMG_WIDTH, FONT_IMG_HEIGHT, GL_ALPHA);

	init_font();
}


void load_texture(int name, unsigned char *texture, int tex_width, int tex_height, int type)
{
  glBindTexture(GL_TEXTURE_2D, name);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (options&HIGH_QUALITY?GL_LINEAR:GL_NEAREST));
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (options&HIGH_QUALITY?GL_LINEAR:GL_NEAREST));
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

  glTexImage2D(GL_TEXTURE_2D, 0, type, tex_width, tex_height,
	       0, type, GL_UNSIGNED_BYTE, texture);
}

GLfloat lpos[]={100, 100, 100, 0};
GLfloat lcolor[]={1, 1, 1, 0};
GLfloat acolor[]={.05, .05, .05, 0};

void resize(int width, int height)
{
  glViewport(0, 0, width, height);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-width/2.0, width/2.0, -height/2.0, height/2.0, CLIP_NEAR, CLIP_FAR);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();


  glTranslatef(0,0,-150);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_POSITION, lpos);
  glLightfv(GL_LIGHT0, GL_AMBIENT, acolor);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, lcolor);
}

void draw_sphere(int lon, int lat)
{
  int i, j;
  float h;
  float x, y, s1, s2, t;

  glEnable(GL_NORMALIZE);

  t=(((float)1/(float)lon)-.5);
  s1=pow(.25-t*t,.5);

  h = .5*(1.0-cos(M_PI/lat));
  glBegin(GL_TRIANGLE_STRIP);
  for(i=0;i<lat+1;i++)
  {
  glTexCoord2f(3*(float)i/lat,0);
  glNormal3f(0,-.5,0);
  glVertex3f(0,.5,0);

    x=cos(i*M_PI*2.0/lat)*s1;
    y=sin(i*M_PI*2.0/lat)*s1;
    glNormal3f(-x,-(.5-h),-y);
    glTexCoord2f( 3*(float)i/lat,1.0/lon);
    glVertex3f(x,.5-h,y);
  }
  glEnd();

  glBegin(GL_TRIANGLE_STRIP);
  for(i=0;i<lat+1;i++)
  {
  glTexCoord2f(3*(float)i/lat,1);
  glNormal3f(0,.5,0);
  glVertex3f(0,-.5,0);

    x=cos(i*M_PI*2.0/lat)*s1;
    y=sin(i*M_PI*2.0/lat)*s1;
    glNormal3f(-x,-(-.5+h),-y);
    glTexCoord2f(3*(float)i/lat,1.0-1.0/lon);
    glVertex3f(x,-.5+h,y);
  }
  glEnd();

  h = 1.0/(float)lon;

  for(j=1;j<lon-1;j++)
  {
    t=(((float)j/(float)lon)-.5);
    s1=pow(.25-t*t,.5);

    t=(((j+1.0)/(float)lon)-.5);
    s2=pow(.25-t*t,.5);

    glBegin(GL_TRIANGLE_STRIP);
    for(i=0;i<lat+1;i++)
    {
      x=cos(i*M_PI*2.0/lat);
      y=sin(i*M_PI*2.0/lat);
      glTexCoord2f(3*(float)i/lat,(float)j/lon);
      glNormal3f(-x*s1,-(.5-(h*j)),-y*s1);
      glVertex3f(x*s1,.5-(h*j),y*s1);
      glTexCoord2f(3*(float)i/lat,(float)(j+1)/lon);
      glNormal3f(-x*s2,-(.5-(h*(j+1))),-y*s2);
      glVertex3f(x*s2,.5-(h*(j+1)),y*s2);
    }
    glEnd();
  }
}

void draw_circle_text(char *s, float radius, float scale)
{
	int i;
	float rot;

	rot=360.0/strlen(s);

	for(i=0;s[i];i++)
	{
		glPushMatrix();
		glRotatef(rot*i,0,0,-1.0);
		glTranslatef(-radius,0,0);

		glScalef(scale, scale, scale);
		glRotatef(90.0,0,0,1.0);
		draw_letter(s[i]);

		glPopMatrix();
	}
}

void draw_logo(int i)
{
	float x,y,dx,dy;

	if(!(options&SHOW_LOGO))
	return;

	y=-.5+.2*sin(i/70.0);
	x=-.5+.2*sin(i/100.0);
	dx=1+.2*sin(i/80.0);
	dy=1+.2*sin(i/90.0);

	glBindTexture(GL_TEXTURE_2D, LOGO_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	glBegin(GL_POLYGON);
	glTexCoord2f(0,1);
	glVertex3f(x,y,0);
	glTexCoord2f(1,1);
	glVertex3f(x+dx,y,0);
	glTexCoord2d(1,0);
	glVertex3f(x+dx,dy+y,0);
	glTexCoord2d(0,0);
	glVertex3f(x,dy+y,0);
	glEnd();
}

static float ftime=0;
static int frame=0;
static double start_time;
float last_fps=0;

void ss_draw(circ_def * inCircleDef,int inCircleCount,double time)
{
	//float fps;
	double now;
	int i;
	//float x, y;
	static int angle=0;
	if(frame==0)
	start_time=time*60;

	frame++;

	now=time*60;

	ftime=120*(float)(now-start_time)/1000.0;

	//printf("%f\n", last_fps);

	glClearColor(0, 0, 0, 0);

	//  glClearColor((sin(color/256.0)+1.0)*.1, (sin(color/128.0)+1.0)*.1, (sin(color/64.0)+1.0)*.1, 0);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	//  glEnable(GL_LIGHTING);
	//  glEnable(GL_DEPTH_TEST);

	/*
	glPushMatrix();
	glRotatef(frame,1,1,0);
	glmDraw(pawn_model, GLM_SMOOTH);
	glPopMatrix();
	*/
	/*
	glPushMatrix();
	glRotatef(frame/3.0,1,.2,.8);
	glScalef(1300,1300,1300);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, MOON_TEXTURE);
	draw_sphere(20,20);
	glPopMatrix();
	*/
	//  glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);


	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_ONE,GL_ONE);

	glPushMatrix();
	glScalef(200,100,100);
	//  draw_logo(ftime);
	glPopMatrix();

	glColor4f((sin(ftime/256.0)+1.0)*.5, (sin(ftime/128.0)+1.0)*.5, (sin(ftime/64.0)+1.0)*.5, (sin(ftime/446.0)+1.0)*.5);

	predraw_string();

	for(i=0;i<inCircleCount;i++)
	{
		float s=(float)inCircleDef[i].scale/100.0;

		glPushMatrix();
		glTranslatef(inCircleDef[i].x,inCircleDef[i].y,0);
		//    glScalef(s,s,s);
		glRotatef(ftime/((float)inCircleDef[i].speed/100.0),0,0,1);
		draw_circle_text(inCircleDef[i].text, inCircleDef[i].radius, s);
		glPopMatrix();
	}

	angle++;
	/*
	glPushMatrix();
	glRotatef(ftime/8.0,0,0,-1);
	draw_circle_text(text_str2, 200, 30);
	glPopMatrix();

	glPushMatrix();
	glScalef(.7,.7,.7);
	glRotatef(ftime/8.0,0,0,1);
	draw_circle_text(text_str3, 200, 30);
	glPopMatrix();
	*/
	/*
	glPushMatrix();
	glScalef(100,100,100);
	glTranslatef(-2,2,0);
	glPushMatrix();
	glRotatef(frame/3.0,1,.2,.8);
	draw_letter('P');
	glPopMatrix();
	glTranslatef(1,0,0);
	glPushMatrix();
	glRotatef(frame/3.0,1,.8,.2);
	draw_letter('A');
	glPopMatrix();
	glTranslatef(1,0,0);
	glPushMatrix();
	glRotatef(frame/3.0,.1,.5,.8);
	draw_letter('W');
	glPopMatrix();
	glTranslatef(1,0,0);
	glPushMatrix();
	glRotatef(frame/3.0,9,.2,.8);
	draw_letter('N');
	glPopMatrix();
	glPopMatrix();
	*/

	postdraw_string();

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);

	glFinish();
	/*if(user_options & OPTION_BUFFER)
	SwapBuffers(hDC);*/
}